from sqlalchemy import Column, Integer, String, Float, ForeignKey, Text, DateTime
from sqlalchemy.orm import relationship
from sqlalchemy.sql import func
from .base import Base


class CarListing(Base):
    __tablename__ = 'car_listings'

    id = Column(Integer, primary_key=True)
    link = Column(Text)
    prepayment_amount = Column(Float)
    engine_capacity = Column(Float)
    transmission = Column(String)
    drive_type = Column(String)
    car_type = Column(String)
    color = Column(String)
    manufacture_year = Column(Integer)
    import_year = Column(Integer)
    engine_type = Column(String)
    interior_color = Column(String)
    leasing = Column(String)
    location = Column(String)
    wheel_drive = Column(String)
    mileage = Column(Integer)
    condition = Column(String)
    doors = Column(Integer)
    description = Column(Text)
    monthly_payment = Column(Float)
    loan_term = Column(Integer)
    price = Column(Float)
    brand = Column(String)
    model = Column(String)
    province = Column(String)
    district = Column(String)
    khoroo = Column(String)
    created_at = Column(DateTime(timezone=True), server_default=func.now())

    images = relationship("CarImage", back_populates="car_listing")

class CarImage(Base):
    __tablename__ = 'car_images'

    id = Column(Integer, primary_key=True)
    car_listing_id = Column(Integer, ForeignKey('car_listings.id'))
    img_url = Column(Text)

    car_listing = relationship("CarListing", back_populates="images")


from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker
from sqlalchemy.engine.url import URL

from contextlib import contextmanager
from . import db_settings

Base = declarative_base()

def db_connect():
    """
    Performs database connection using database settings from settings.py.
    Returns sqlalchemy engine instance
    """
    return create_engine(URL.create(**db_settings.DATABASE))

SessionFactory = sessionmaker(bind=db_connect())

@contextmanager
def session_scope():
    """
    Provide a transactional scope around a series of operations.
    """
    session = SessionFactory()
    try:
        yield session
        session.commit()
    except Exception as e:
        session.rollback()
        raise e
    finally:
        session.close()



from sqlalchemy import Column, Integer, String, Float, ForeignKey, Text, DateTime
from sqlalchemy.orm import relationship
from sqlalchemy.sql import func
from .base import Base


class Properties(Base):
    __tablename__ = "properties"
    id = Column(Integer, primary_key=True)
    link = Column(String)
    rooms = Column('rooms', Integer)
    garage = Column('garage', String)
    balcony_number = Column('balconyNumber', Integer)
    area = Column('area', Float)
    location = Column('location', String)
    door = Column('door', String)
    window = Column('window', String)
    floor = Column('floor', String)
    window_number = Column('windowNumber', Integer)
    building_floor = Column('buildingFloor', Integer)
    which_floor = Column('whichFloor', Integer)
    commission_year = Column('commissionYear', Integer)
    leasing = Column('leasing', String)
    progress = Column('progress', String)
    price = Column('price', Float)
    province = Column('province', String)
    district = Column('district', String)
    khoroo = Column('khoroo', String)
    property_type = Column('propertyType', String)
    sell_type = Column('sellType', String)
    created_at = Column(DateTime(timezone=True), server_default=func.now())

    images = relationship("PropertyImage", back_populates="property")

class PropertyImage(Base):
    __tablename__ = 'property_images'

    id = Column(Integer, primary_key=True)
    property_id = Column(Integer, ForeignKey('properties.id'))
    img_url = Column(Text)

    property = relationship("Properties", back_populates="images")
